@@ -42,41 +42,48 @@
 #include "RequestGroup.h"
 #include "DownloadContext.h"
 #include "Command.h"
-#include "RequestGroupMan.h"
-#include "CheckIntegrityEntry.h"
-#include "ServerStatMan.h"
+#include "PeerStat.h"
+#include "FileEntry.h"
+#include "PieceStorage.h"
+#include "DiskAdaptor.h"
+#include "LogFactory.h"
 
 namespace aria2 {
 
-StreamFileAllocationEntry::StreamFileAllocationEntry(RequestGroup* requestGroup,
-                                                     Command* nextCommand):
-  FileAllocationEntry(requestGroup, nextCommand)
-{}
-
-StreamFileAllocationEntry::~StreamFileAllocationEntry() {}
-
-void StreamFileAllocationEntry::prepareForNextAction
-(std::vector<Command*>& commands,
- DownloadEngine* e)
+StreamFileAllocationEntry::StreamFileAllocationEntry(
+    RequestGroup* requestGroup, std::unique_ptr<Command> nextCommand)
+    : FileAllocationEntry(requestGroup, std::move(nextCommand))
 {
+}
+
+StreamFileAllocationEntry::~StreamFileAllocationEntry() = default;
+
+void StreamFileAllocationEntry::prepareForNextAction(
+    std::vector<std::unique_ptr<Command>>& commands, DownloadEngine* e)
+{
+  auto rg = getRequestGroup();
+  auto& dctx = rg->getDownloadContext();
+  auto& ps = rg->getPieceStorage();
+  auto diskAdaptor = ps->getDiskAdaptor();
+  auto& option = rg->getOption();
+
   // For DownloadContext::resetDownloadStartTime(), see also
   // RequestGroup::createInitialCommand()
-  getRequestGroup()->getDownloadContext()->resetDownloadStartTime();
-  if(getNextCommand()) {
+  dctx->resetDownloadStartTime();
+  if (option->getAsBool(PREF_ENABLE_MMAP) &&
+      option->get(PREF_FILE_ALLOCATION) != V_NONE &&
+      diskAdaptor->size() <= option->getAsLLInt(PREF_MAX_MMAP_LIMIT)) {
+    diskAdaptor->enableMmap();
+  }
+  if (getNextCommand()) {
     // Reset download start time of PeerStat because it is started
     // before file allocation begins.
-    const SharedHandle<DownloadContext>& dctx =
-      getRequestGroup()->getDownloadContext();
-    const std::vector<SharedHandle<FileEntry> >& fileEntries =
-      dctx->getFileEntries();
-    for(std::vector<SharedHandle<FileEntry> >::const_iterator i =
-          fileEntries.begin(), eoi = fileEntries.end(); i != eoi; ++i) {
-      const std::deque<SharedHandle<Request> >& reqs =
-        (*i)->getInFlightRequests();
-      for(std::deque<SharedHandle<Request> >::const_iterator j =
-            reqs.begin(), eoj = reqs.end(); j != eoj; ++j) {
-        const SharedHandle<PeerStat>& peerStat = (*j)->getPeerStat();
-        if(!peerStat.isNull()) {
+    const auto& fileEntries = dctx->getFileEntries();
+    for (auto& f : fileEntries) {
+      const auto& reqs = f->getInFlightRequests();
+      for (auto& req : reqs) {
+        const auto& peerStat = req->getPeerStat();
+        if (peerStat) {
           peerStat->downloadStart();
         }
       }
@@ -86,9 +93,19 @@ void StreamFileAllocationEntry::prepareForNextAction
     e->setNoWait(true);
     commands.push_back(popNextCommand());
     // try remaining uris
-    getRequestGroup()->createNextCommandWithAdj(commands, e, -1);
-  } else {
-    getRequestGroup()->createNextCommandWithAdj(commands, e, 0);
+    rg->createNextCommandWithAdj(commands, e, -1);
+  }
+  else {
+    rg->createNextCommandWithAdj(commands, e, 0);
+  }
+
+  if (!rg->allDownloadFinished()) {
+    try {
+      rg->saveControlFile();
+    }
+    catch (RecoverableException& e) {
+      A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, e);
+    }
   }
 }
 
