@@ -61,20 +61,21 @@
 #include "MetadataInfo.h"
 #include "OptionParser.h"
 #include "SegList.h"
+#include "download_handlers.h"
+#include "SimpleRandomizer.h"
 #ifdef ENABLE_BITTORRENT
-#  include "bittorrent_helper.h"
-#  include "BtConstants.h"
-#  include "UTMetadataPostDownloadHandler.h"
-#  include "ValueBaseBencodeParser.h"
+#include "bittorrent_helper.h"
+#include "BtConstants.h"
+#include "ValueBaseBencodeParser.h"
 #endif // ENABLE_BITTORRENT
 
 namespace aria2 {
 
 namespace {
-void unfoldURI
-(std::vector<std::string>& result, const std::vector<std::string>& args)
+void unfoldURI(std::vector<std::string>& result,
+               const std::vector<std::string>& args)
 {
-  for(const auto& i : args) {
+  for (const auto& i : args) {
     paramed_string::expand(std::begin(i), std::end(i),
                            std::back_inserter(result));
   }
@@ -82,23 +83,21 @@ void unfoldURI
 } // namespace
 
 namespace {
-template<typename InputIterator>
-void splitURI(std::vector<std::string>& result,
-              InputIterator begin,
-              InputIterator end,
-              size_t numSplit,
-              size_t maxIter)
+template <typename InputIterator>
+void splitURI(std::vector<std::string>& result, InputIterator begin,
+              InputIterator end, size_t numSplit, size_t maxIter)
 {
   size_t numURIs = std::distance(begin, end);
-  if(numURIs >= numSplit) {
+  if (numURIs >= numSplit) {
     result.insert(std::end(result), begin, end);
-  } else if(numURIs > 0) {
-    size_t num = std::min(numSplit/numURIs, maxIter);
-    for(size_t i = 0; i < num; ++i) {
+  }
+  else if (numURIs > 0) {
+    size_t num = std::min(numSplit / numURIs, maxIter);
+    for (size_t i = 0; i < num; ++i) {
       result.insert(std::end(result), begin, end);
     }
-    if(num < maxIter) {
-      result.insert(std::end(result), begin, begin+(numSplit%numURIs));
+    if (num < maxIter) {
+      result.insert(std::end(result), begin, begin + (numSplit % numURIs));
     }
   }
 }
@@ -108,18 +107,19 @@ namespace {
 std::shared_ptr<GroupId> getGID(const std::shared_ptr<Option>& option)
 {
   std::shared_ptr<GroupId> gid;
-  if(option->defined(PREF_GID)) {
+  if (option->defined(PREF_GID)) {
     a2_gid_t n;
-    if(GroupId::toNumericId(n, option->get(PREF_GID).c_str()) != 0) {
-      throw DL_ABORT_EX(fmt("%s is invalid for GID.",
-                            option->get(PREF_GID).c_str()));
+    if (GroupId::toNumericId(n, option->get(PREF_GID).c_str()) != 0) {
+      throw DL_ABORT_EX(
+          fmt("%s is invalid for GID.", option->get(PREF_GID).c_str()));
     }
     gid = GroupId::import(n);
-    if(!gid) {
-      throw DL_ABORT_EX(fmt("GID %s is not unique.",
-                            option->get(PREF_GID).c_str()));
+    if (!gid) {
+      throw DL_ABORT_EX(
+          fmt("GID %s is not unique.", option->get(PREF_GID).c_str()));
     }
-  } else {
+  }
+  else {
     gid = GroupId::create();
   }
   return gid;
@@ -127,25 +127,23 @@ std::shared_ptr<GroupId> getGID(const std::shared_ptr<Option>& option)
 } // namespace
 
 namespace {
-std::shared_ptr<RequestGroup> createRequestGroup
-(const std::shared_ptr<Option>& optionTemplate,
- const std::vector<std::string>& uris,
- bool useOutOption = false)
+std::shared_ptr<RequestGroup>
+createRequestGroup(const std::shared_ptr<Option>& optionTemplate,
+                   const std::vector<std::string>& uris,
+                   bool useOutOption = false)
 {
   auto option = util::copy(optionTemplate);
   auto rg = std::make_shared<RequestGroup>(getGID(option), option);
-  auto dctx = std::make_shared<DownloadContext>
-    (option->getAsInt(PREF_PIECE_LENGTH),
-     0,
-     useOutOption&&!option->blank(PREF_OUT)?
-     util::applyDir(option->get(PREF_DIR),
-                    option->get(PREF_OUT)):A2STR::NIL);
+  auto dctx = std::make_shared<DownloadContext>(
+      option->getAsInt(PREF_PIECE_LENGTH), 0,
+      useOutOption && !option->blank(PREF_OUT)
+          ? util::applyDir(option->get(PREF_DIR), option->get(PREF_OUT))
+          : A2STR::NIL);
   dctx->getFirstFileEntry()->setUris(uris);
-  dctx->getFirstFileEntry()->setMaxConnectionPerServer
-    (option->getAsInt(PREF_MAX_CONNECTION_PER_SERVER));
-#ifdef ENABLE_MESSAGE_DIGEST
+  dctx->getFirstFileEntry()->setMaxConnectionPerServer(
+      option->getAsInt(PREF_MAX_CONNECTION_PER_SERVER));
   const std::string& checksum = option->get(PREF_CHECKSUM);
-  if(!checksum.empty()) {
+  if (!checksum.empty()) {
     auto p = util::divide(std::begin(checksum), std::end(checksum), '=');
     std::string hashType(p.first.first, p.first.second);
     std::string hexDigest(p.second.first, p.second.second);
@@ -153,18 +151,21 @@ std::shared_ptr<RequestGroup> createRequestGroup
     dctx->setDigest(hashType,
                     util::fromHex(std::begin(hexDigest), std::end(hexDigest)));
   }
-#endif // ENABLE_MESSAGE_DIGEST
   rg->setDownloadContext(dctx);
-  rg->setPauseRequested(option->getAsBool(PREF_PAUSE));
+
+  if (option->getAsBool(PREF_ENABLE_RPC)) {
+    rg->setPauseRequested(option->getAsBool(PREF_PAUSE));
+  }
+
   removeOneshotOption(option);
   return rg;
 }
 } // namespace
 
-#if defined ENABLE_BITTORRENT || ENABLE_METALINK
+#if defined(ENABLE_BITTORRENT) || defined(ENABLE_METALINK)
 namespace {
-std::shared_ptr<MetadataInfo> createMetadataInfo
-(const std::shared_ptr<GroupId>& gid, const std::string& uri)
+std::shared_ptr<MetadataInfo>
+createMetadataInfo(const std::shared_ptr<GroupId>& gid, const std::string& uri)
 {
   return std::make_shared<MetadataInfo>(gid, uri);
 }
@@ -185,8 +186,7 @@ std::shared_ptr<RequestGroup>
 createBtRequestGroup(const std::string& metaInfoUri,
                      const std::shared_ptr<Option>& optionTemplate,
                      const std::vector<std::string>& auxUris,
-                     const ValueBase* torrent,
-                     bool adjustAnnounceUri = true)
+                     const ValueBase* torrent, bool adjustAnnounceUri = true)
 {
   auto option = util::copy(optionTemplate);
   auto gid = getGID(option);
@@ -195,26 +195,35 @@ createBtRequestGroup(const std::string& metaInfoUri,
   // may throw exception
   bittorrent::loadFromMemory(torrent, dctx, option, auxUris,
                              metaInfoUri.empty() ? "default" : metaInfoUri);
-  if(metaInfoUri.empty()) {
+  for (auto& fe : dctx->getFileEntries()) {
+    auto& uris = fe->getRemainingUris();
+    std::shuffle(std::begin(uris), std::end(uris),
+                 *SimpleRandomizer::getInstance());
+  }
+  if (metaInfoUri.empty()) {
     rg->setMetadataInfo(createMetadataInfoDataOnly());
-  } else {
+  }
+  else {
     rg->setMetadataInfo(createMetadataInfo(gid, metaInfoUri));
   }
-  if(adjustAnnounceUri) {
+  if (adjustAnnounceUri) {
     bittorrent::adjustAnnounceUri(bittorrent::getTorrentAttrs(dctx), option);
   }
-  SegList<int> sgl;
-  util::parseIntSegments(sgl, option->get(PREF_SELECT_FILE));
+  auto sgl = util::parseIntSegments(option->get(PREF_SELECT_FILE));
   sgl.normalize();
-  dctx->setFileFilter(sgl);
+  dctx->setFileFilter(std::move(sgl));
   std::istringstream indexOutIn(option->get(PREF_INDEX_OUT));
   auto indexPaths = util::createIndexPaths(indexOutIn);
-  for(const auto& i : indexPaths) {
-    dctx->setFilePathWithIndex
-      (i.first, util::applyDir(option->get(PREF_DIR), i.second));
+  for (const auto& i : indexPaths) {
+    dctx->setFilePathWithIndex(i.first,
+                               util::applyDir(option->get(PREF_DIR), i.second));
   }
   rg->setDownloadContext(dctx);
-  rg->setPauseRequested(option->getAsBool(PREF_PAUSE));
+
+  if (option->getAsBool(PREF_ENABLE_RPC)) {
+    rg->setPauseRequested(option->getAsBool(PREF_PAUSE));
+  }
+
   // Remove "metalink" from Accept Type list to avoid server from
   // responding Metalink file for web-seeding URIs.
   dctx->setAcceptMetalink(false);
@@ -225,15 +234,14 @@ createBtRequestGroup(const std::string& metaInfoUri,
 
 namespace {
 std::shared_ptr<RequestGroup>
-createBtMagnetRequestGroup
-(const std::string& magnetLink,
- const std::shared_ptr<Option>& optionTemplate)
+createBtMagnetRequestGroup(const std::string& magnetLink,
+                           const std::shared_ptr<Option>& optionTemplate)
 {
   auto option = util::copy(optionTemplate);
   auto gid = getGID(option);
   auto rg = std::make_shared<RequestGroup>(gid, option);
-  auto dctx = std::make_shared<DownloadContext>(METADATA_PIECE_SIZE, 0,
-                                                A2STR::NIL);
+  auto dctx = std::make_shared<DownloadContext>(METADATA_PIECE_SIZE, 0);
+
   // We only know info hash. Total Length is unknown at this moment.
   dctx->markTotalLengthIsUnknown();
   rg->setFileAllocationEnabled(false);
@@ -241,64 +249,68 @@ createBtMagnetRequestGroup
   bittorrent::loadMagnet(magnetLink, dctx);
   auto torrentAttrs = bittorrent::getTorrentAttrs(dctx);
   bittorrent::adjustAnnounceUri(torrentAttrs, option);
-  dctx->getFirstFileEntry()->setPath(torrentAttrs->name);
+  // torrentAttrs->name may contain "/", but we use basename of
+  // FileEntry::getPath() to print out in-memory download entry.
+  // Since "/" is treated as separator, we replace it with "-".
+  dctx->getFirstFileEntry()->setPath(
+      util::replace(torrentAttrs->name, "/", "-"));
   rg->setDownloadContext(dctx);
   rg->clearPostDownloadHandler();
-  auto utMetadataPostHandler =
-    std::make_shared<UTMetadataPostDownloadHandler>();
-  rg->addPostDownloadHandler(utMetadataPostHandler);
+  rg->addPostDownloadHandler(
+      download_handlers::getUTMetadataPostDownloadHandler());
   rg->setDiskWriterFactory(std::make_shared<ByteArrayDiskWriterFactory>());
   rg->setMetadataInfo(createMetadataInfo(gid, magnetLink));
   rg->markInMemoryDownload();
-  rg->setPauseRequested(option->getAsBool(PREF_PAUSE));
+
+  if (option->getAsBool(PREF_ENABLE_RPC)) {
+    rg->setPauseRequested(option->getAsBool(PREF_PAUSE));
+  }
+
   removeOneshotOption(option);
   return rg;
 }
 } // namespace
 
-void createRequestGroupForBitTorrent
-(std::vector<std::shared_ptr<RequestGroup>>& result,
- const std::shared_ptr<Option>& option,
- const std::vector<std::string>& uris,
- const std::string& metaInfoUri,
- const std::string& torrentData,
- bool adjustAnnounceUri)
+void createRequestGroupForBitTorrent(
+    std::vector<std::shared_ptr<RequestGroup>>& result,
+    const std::shared_ptr<Option>& option, const std::vector<std::string>& uris,
+    const std::string& metaInfoUri, const std::string& torrentData,
+    bool adjustAnnounceUri)
 {
   std::unique_ptr<ValueBase> torrent;
   bittorrent::ValueBaseBencodeParser parser;
-  if(torrentData.empty()) {
+  if (torrentData.empty()) {
     torrent = parseFile(parser, metaInfoUri);
-  } else {
-    ssize_t error;
-    torrent = parser.parseFinal(torrentData.c_str(), torrentData.size(),
-                                error);
   }
-  if(!torrent) {
+  else {
+    ssize_t error;
+    torrent = parser.parseFinal(torrentData.c_str(), torrentData.size(), error);
+  }
+  if (!torrent) {
     throw DL_ABORT_EX2("Bencode decoding failed",
                        error_code::BENCODE_PARSE_ERROR);
   }
   createRequestGroupForBitTorrent(result, option, uris, metaInfoUri,
-                                  torrent.get());
+                                  torrent.get(), adjustAnnounceUri);
 }
 
-void createRequestGroupForBitTorrent
-(std::vector<std::shared_ptr<RequestGroup>>& result,
- const std::shared_ptr<Option>& option,
- const std::vector<std::string>& uris,
- const std::string& metaInfoUri,
- const ValueBase* torrent,
- bool adjustAnnounceUri)
+void createRequestGroupForBitTorrent(
+    std::vector<std::shared_ptr<RequestGroup>>& result,
+    const std::shared_ptr<Option>& option, const std::vector<std::string>& uris,
+    const std::string& metaInfoUri, const ValueBase* torrent,
+    bool adjustAnnounceUri)
 {
   std::vector<std::string> nargs;
-  if(option->get(PREF_PARAMETERIZED_URI) == A2_V_TRUE) {
+  if (option->get(PREF_PARAMETERIZED_URI) == A2_V_TRUE) {
     unfoldURI(nargs, uris);
-  } else {
+  }
+  else {
     nargs = uris;
   }
   // we ignore -Z option here
   size_t numSplit = option->getAsInt(PREF_SPLIT);
-  auto rg = createBtRequestGroup(metaInfoUri, option, nargs,
-                                 torrent, adjustAnnounceUri);
+  auto rg = createBtRequestGroup(metaInfoUri, option, nargs, torrent,
+                                 adjustAnnounceUri);
   rg->setNumConcurrentCommand(numSplit);
   result.push_back(rg);
 }
@@ -306,17 +318,16 @@ void createRequestGroupForBitTorrent
 #endif // ENABLE_BITTORRENT
 
 #ifdef ENABLE_METALINK
-void createRequestGroupForMetalink
-(std::vector<std::shared_ptr<RequestGroup>>& result,
- const std::shared_ptr<Option>& option,
- const std::string& metalinkData)
+void createRequestGroupForMetalink(
+    std::vector<std::shared_ptr<RequestGroup>>& result,
+    const std::shared_ptr<Option>& option, const std::string& metalinkData)
 {
-  if(metalinkData.empty()) {
-    Metalink2RequestGroup().generate(result,
-                                     option->get(PREF_METALINK_FILE),
+  if (metalinkData.empty()) {
+    Metalink2RequestGroup().generate(result, option->get(PREF_METALINK_FILE),
                                      option,
                                      option->get(PREF_METALINK_BASE_URI));
-  } else {
+  }
+  else {
     auto dw = std::make_shared<ByteArrayDiskWriter>();
     dw->setString(metalinkData);
     Metalink2RequestGroup().generate(result, dw, option,
@@ -333,25 +344,26 @@ private:
   std::shared_ptr<Option> option_;
   bool ignoreLocalPath_;
   bool throwOnError_;
+
 public:
   AccRequestGroup(std::vector<std::shared_ptr<RequestGroup>>& requestGroups,
-                  const std::shared_ptr<Option>& option,
-                  bool ignoreLocalPath = false,
-                  bool throwOnError = false):
-    requestGroups_(requestGroups), option_(option),
-    ignoreLocalPath_(ignoreLocalPath),
-    throwOnError_(throwOnError)
-  {}
-
-  void
-  operator()(const std::string& uri)
+                  std::shared_ptr<Option> option, bool ignoreLocalPath = false,
+                  bool throwOnError = false)
+      : requestGroups_(requestGroups),
+        option_(std::move(option)),
+        ignoreLocalPath_(ignoreLocalPath),
+        throwOnError_(throwOnError)
   {
-    if(detector_.isStreamProtocol(uri)) {
+  }
+
+  void operator()(const std::string& uri)
+  {
+    if (detector_.isStreamProtocol(uri)) {
       std::vector<std::string> streamURIs;
       size_t numIter = option_->getAsInt(PREF_MAX_CONNECTION_PER_SERVER);
       size_t numSplit = option_->getAsInt(PREF_SPLIT);
       size_t num = std::min(numIter, numSplit);
-      for(size_t i = 0; i < num; ++i) {
+      for (size_t i = 0; i < num; ++i) {
         streamURIs.push_back(uri);
       }
       auto rg = createRequestGroup(option_, streamURIs);
@@ -359,22 +371,25 @@ public:
       requestGroups_.push_back(rg);
     }
 #ifdef ENABLE_BITTORRENT
-    else if(detector_.guessTorrentMagnet(uri)) {
+    else if (detector_.guessTorrentMagnet(uri)) {
       requestGroups_.push_back(createBtMagnetRequestGroup(uri, option_));
-    } else if(!ignoreLocalPath_ && detector_.guessTorrentFile(uri)) {
+    }
+    else if (!ignoreLocalPath_ && detector_.guessTorrentFile(uri)) {
       try {
         bittorrent::ValueBaseBencodeParser parser;
         auto torrent = parseFile(parser, uri);
-        if(!torrent) {
+        if (!torrent) {
           throw DL_ABORT_EX2("Bencode decoding failed",
                              error_code::BENCODE_PARSE_ERROR);
         }
-        requestGroups_.push_back
-          (createBtRequestGroup(uri, option_, {}, torrent.get()));
-      } catch(RecoverableException& e) {
-        if(throwOnError_) {
+        requestGroups_.push_back(
+            createBtRequestGroup(uri, option_, {}, torrent.get()));
+      }
+      catch (RecoverableException& e) {
+        if (throwOnError_) {
           throw;
-        } else {
+        }
+        else {
           // error occurred while parsing torrent file.
           // We simply ignore it.
           A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, e);
@@ -383,14 +398,16 @@ public:
     }
 #endif // ENABLE_BITTORRENT
 #ifdef ENABLE_METALINK
-    else if(!ignoreLocalPath_ && detector_.guessMetalinkFile(uri)) {
+    else if (!ignoreLocalPath_ && detector_.guessMetalinkFile(uri)) {
       try {
         Metalink2RequestGroup().generate(requestGroups_, uri, option_,
                                          option_->get(PREF_METALINK_BASE_URI));
-      } catch(RecoverableException& e) {
-        if(throwOnError_) {
+      }
+      catch (RecoverableException& e) {
+        if (throwOnError_) {
           throw;
-        } else {
+        }
+        else {
           // error occurred while parsing metalink file.
           // We simply ignore it.
           A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, e);
@@ -399,9 +416,10 @@ public:
     }
 #endif // ENABLE_METALINK
     else {
-      if(throwOnError_) {
+      if (throwOnError_) {
         throw DL_ABORT_EX(fmt(MSG_UNRECOGNIZED_URI, uri.c_str()));
-      } else {
+      }
+      else {
         A2_LOG_ERROR(fmt(MSG_UNRECOGNIZED_URI, uri.c_str()));
       }
     }
@@ -413,38 +431,38 @@ namespace {
 class StreamProtocolFilter {
 private:
   ProtocolDetector detector_;
+
 public:
-  bool operator()(const std::string& uri) {
+  bool operator()(const std::string& uri)
+  {
     return detector_.isStreamProtocol(uri);
   }
 };
 } // namespace
 
-void createRequestGroupForUri
-(std::vector<std::shared_ptr<RequestGroup>>& result,
- const std::shared_ptr<Option>& option,
- const std::vector<std::string>& uris,
- bool ignoreForceSequential,
- bool ignoreLocalPath,
- bool throwOnError)
+void createRequestGroupForUri(
+    std::vector<std::shared_ptr<RequestGroup>>& result,
+    const std::shared_ptr<Option>& option, const std::vector<std::string>& uris,
+    bool ignoreForceSequential, bool ignoreLocalPath, bool throwOnError)
 {
   std::vector<std::string> nargs;
-  if(option->get(PREF_PARAMETERIZED_URI) == A2_V_TRUE) {
+  if (option->get(PREF_PARAMETERIZED_URI) == A2_V_TRUE) {
     unfoldURI(nargs, uris);
-  } else {
+  }
+  else {
     nargs = uris;
   }
-  if(!ignoreForceSequential &&
-     option->get(PREF_FORCE_SEQUENTIAL) == A2_V_TRUE) {
-    std::for_each(std::begin(nargs), std::end(nargs),
-                  AccRequestGroup(result, option, ignoreLocalPath,
-                                  throwOnError));
-  } else {
-    auto strmProtoEnd =
-      std::stable_partition(std::begin(nargs), std::end(nargs),
-                            StreamProtocolFilter());
+  if (!ignoreForceSequential &&
+      option->get(PREF_FORCE_SEQUENTIAL) == A2_V_TRUE) {
+    std::for_each(
+        std::begin(nargs), std::end(nargs),
+        AccRequestGroup(result, option, ignoreLocalPath, throwOnError));
+  }
+  else {
+    auto strmProtoEnd = std::stable_partition(
+        std::begin(nargs), std::end(nargs), StreamProtocolFilter());
     // let's process http/ftp protocols first.
-    if(std::begin(nargs) != strmProtoEnd) {
+    if (std::begin(nargs) != strmProtoEnd) {
       size_t numIter = option->getAsInt(PREF_MAX_CONNECTION_PER_SERVER);
       size_t numSplit = option->getAsInt(PREF_SPLIT);
       std::vector<std::string> streamURIs;
@@ -453,50 +471,51 @@ void createRequestGroupForUri
         auto rg = createRequestGroup(option, streamURIs, true);
         rg->setNumConcurrentCommand(numSplit);
         result.push_back(rg);
-      } catch(RecoverableException& e) {
-        if(throwOnError) {
+      }
+      catch (RecoverableException& e) {
+        if (throwOnError) {
           throw;
-        } else {
+        }
+        else {
           A2_LOG_ERROR_EX(EX_EXCEPTION_CAUGHT, e);
         }
       }
     }
     // process remaining URIs(local metalink, BitTorrent files)
-    std::for_each(strmProtoEnd, std::end(nargs),
-                  AccRequestGroup(result, option, ignoreLocalPath,
-                                  throwOnError));
+    std::for_each(
+        strmProtoEnd, std::end(nargs),
+        AccRequestGroup(result, option, ignoreLocalPath, throwOnError));
   }
 }
 
-bool createRequestGroupFromUriListParser
-(std::vector<std::shared_ptr<RequestGroup>>& result,
- const Option* option,
- UriListParser* uriListParser)
+bool createRequestGroupFromUriListParser(
+    std::vector<std::shared_ptr<RequestGroup>>& result, const Option* option,
+    UriListParser* uriListParser)
 {
   // Since result already contains some entries, we cache the size of
   // it. Later, we use this value to determine RequestGroup is
   // actually created.
   size_t num = result.size();
-  while(uriListParser->hasNext()) {
+  while (uriListParser->hasNext()) {
     std::vector<std::string> uris;
     Option tempOption;
     uriListParser->parseNext(uris, tempOption);
-    if(uris.empty()) {
+    if (uris.empty()) {
       continue;
     }
     auto requestOption = std::make_shared<Option>(*option);
     requestOption->remove(PREF_OUT);
     const auto& oparser = OptionParser::getInstance();
-    for(size_t i = 1, len = option::countOption(); i < len; ++i) {
+    for (size_t i = 1, len = option::countOption(); i < len; ++i) {
       auto pref = option::i2p(i);
       auto h = oparser->find(pref);
-      if(h && h->getInitialOption() && tempOption.defined(pref)) {
+      if (h && h->getInitialOption() && tempOption.defined(pref)) {
         requestOption->put(pref, tempOption.get(pref));
       }
     }
     // This does not throw exception because throwOnError = false.
     createRequestGroupForUri(result, requestOption, uris);
-    if(num < result.size()) {
+    if (num < result.size()) {
       return true;
     }
   }
@@ -506,38 +525,37 @@ bool createRequestGroupFromUriListParser
 std::shared_ptr<UriListParser> openUriListParser(const std::string& filename)
 {
   std::string listPath;
-  if(filename == "-") {
-    listPath = DEV_STDIN;
-  } else {
-    if(!File(filename).isFile()) {
-      throw DL_ABORT_EX
-        (fmt(EX_FILE_OPEN, filename.c_str(), "No such file"));
-    }
-    listPath = filename;
+
+  auto f = File(filename);
+  if (!f.exists() || f.isDir()) {
+    throw DL_ABORT_EX(fmt(EX_FILE_OPEN, filename.c_str(),
+                          "File not found or it is a directory"));
   }
+  listPath = filename;
+
   return std::make_shared<UriListParser>(listPath);
 }
 
-void createRequestGroupForUriList
-(std::vector<std::shared_ptr<RequestGroup>>& result,
- const std::shared_ptr<Option>& option)
+void createRequestGroupForUriList(
+    std::vector<std::shared_ptr<RequestGroup>>& result,
+    const std::shared_ptr<Option>& option)
 {
   auto uriListParser = openUriListParser(option->get(PREF_INPUT_FILE));
-  while(createRequestGroupFromUriListParser(result, option.get(),
-                                            uriListParser.get()));
+  while (createRequestGroupFromUriListParser(result, option.get(),
+                                             uriListParser.get()))
+    ;
 }
 
-std::shared_ptr<MetadataInfo>
-createMetadataInfoFromFirstFileEntry
-(const std::shared_ptr<GroupId>& gid,
- const std::shared_ptr<DownloadContext>& dctx)
+std::shared_ptr<MetadataInfo> createMetadataInfoFromFirstFileEntry(
+    const std::shared_ptr<GroupId>& gid,
+    const std::shared_ptr<DownloadContext>& dctx)
 {
-  if(dctx->getFileEntries().empty()) {
+  if (dctx->getFileEntries().empty()) {
     return nullptr;
-  } else {
-    std::vector<std::string> uris;
-    dctx->getFileEntries()[0]->getUris(uris);
-    if(uris.empty()) {
+  }
+  else {
+    auto uris = dctx->getFileEntries()[0]->getUris();
+    if (uris.empty()) {
       return nullptr;
     }
     return std::make_shared<MetadataInfo>(gid, uris[0]);
